import { Choice, Map, Parallel, State, StateMachine, Task, Wait } from 'asl-types';
import Serverless from 'serverless';
import PluginManager from 'serverless/classes/PluginManager';
import Service from 'serverless/classes/Service';
declare type RequireAtLeastOne<T, Keys extends keyof T = keyof T> = Pick<T, Exclude<keyof T, Keys>> & {
    [K in Keys]-?: Required<Pick<T, K>> & Partial<Pick<T, Exclude<Keys, K>>>;
}[Keys];
export declare type StateMachineBase = {
    definition: StateMachine;
};
export declare type Maybe<T> = null | undefined | T;
export declare type Event = Record<string, any>;
export declare type NotCompletedState = Parallel | Task | Map | Wait | Choice;
export declare const isNotCompletedState: (state: State) => state is NotCompletedState;
export declare type StateValueReturn = void | Promise<'Fail' | 'Succeed'> | StateHandler | ChoiceConditional;
export declare type ChoiceInstance = {
    variable: string;
    condition: Partial<Choice>;
    choiceFunction?: string;
    compareWithValue: unknown;
    checkFunction: <T>(value: T, secondValue: T) => boolean;
};
export declare type ChoiceConditional = {
    choice: ChoiceInstance[];
    defaultFunction?: string;
};
export declare type Branch = StateMachine;
export declare const notEmpty: <TValue>(value: TValue) => value is TValue;
interface StateType {
    Type: string;
}
export declare const isType: (type: string) => <T extends StateType>(state: State | T) => state is T;
export declare const definitionIsHandler: (value: Maybe<Serverless.FunctionDefinitionHandler | Serverless.FunctionDefinitionImage>) => value is Serverless.FunctionDefinitionHandler;
export declare const stateIsChoiceConditional: (value: Maybe<ChoiceConditional | StateHandler>) => value is ChoiceConditional;
export declare type Failure = {
    Cause?: unknown;
    Error?: unknown;
};
export declare type Callback = (event: any, context: ContextObject, done: ContextObject['cb']) => void;
export declare type AsyncCallback = (event: any, context: ContextObject, done?: ContextObject['cb']) => Promise<void | Event>;
export declare type StateHandler = {
    waitState?: boolean;
    name?: string;
    f: (event: Event) => Callback | Promise<void | AsyncCallback>;
    choice?: Choice;
};
export declare type ContextObject = {
    name: string;
    attempt: number;
    cb: (err: Maybe<Error>, result?: Event) => void | Callback | Promise<void | Callback>;
    done: ContextObject['cb'];
    succeed: (result: Event) => void;
    fail: (err: Error) => void;
};
export declare type Options = {
    location?: Maybe<string>;
    stateMachine?: Maybe<string>;
    lambdaEndpoint?: Maybe<string>;
    detailedLog?: boolean;
    l?: boolean;
    [key: string]: unknown;
    stage?: string;
    region?: string;
} & RequireAtLeastOne<{
    eventFile?: Maybe<string>;
    event?: Maybe<Event>;
    e?: Options['event'];
    ef?: Options['eventFile'];
}, 'event' | 'e'>;
export declare class SLSError extends Error {
    statusCode?: string;
    constructor(msg: string, statusCode?: string);
}
export declare type ServerlessWithError = Serverless & {
    classes: {
        Error: typeof SLSError;
    };
    pluginManager: PluginManager & {
        cliOptions: {
            stage?: string;
            region?: string;
        };
    };
    service: Service & {
        stepFunctions?: {
            activities?: [];
            stateMachines?: {
                [key: string]: StateMachineBase;
            };
        };
    };
};
export {};
